@@ -23,7 +23,7 @@ sub load_data {
 		    }
     }
 
-  Set::Array->new(@data);
+    \@data
 }
 
 
